// EdgeFile.cpp: CEdgeFile NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EdgeFile.h"
#include "EdgeCompress.h"

#include <string.h>
#include <stdio.h>

// RXgN^
CEdgeFile::CEdgeFile()
{
	int i;
	for (i = 0; i < LAYER_MAX; i++) {
		m_apLayer[i] = NULL;
	}
	Clean();
}

// fXgN^
CEdgeFile::~CEdgeFile()
{
	Clean();
}

// f[^N[
void CEdgeFile::Clean()
{
	int i;
	for (i = 0; i < LAYER_MAX; i++) {
		if (m_apLayer[i]) {
			delete m_apLayer[i];
			m_apLayer[i] = NULL;
		}
	}

	m_wLayer = 0;
	m_nWidth = 0;
	m_nHeight = 0;
	m_btTransColor = 0;

	ZeroMemory(m_acolPalette, 256 * 3);
}

// C[W̃TCYݒ
bool CEdgeFile::SetImageSize(int nWidth, int nHeight)
{
	if (nWidth <= 0 || nWidth > IMAGE_WIDTH_MAX) {
		return false;
	}
	if (nHeight <= 0 || nHeight > IMAGE_HEIGHT_MAX) {
		return false;
	}

	m_nWidth = nWidth;
	m_nHeight = nHeight;

	return true;
}

// ǂݍݎs
int CEdgeFile::Read(const char *pszFilename)
{
	Clean();

	// t@CI[v
	FILE* fp = fopen(pszFilename, "rb");
	if (fp == NULL) {
		return EDGELIB_CANNOT_OPEN;
	}

	// t@Cwb_[ǂݍ
	char szHeader[10];
	if (fread(szHeader, 1, 10, fp) != 10) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}
	
	// EDGEt@C̊mF
	if (strcmp(szHeader, "EDGE")) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// ǂݍ
	if (fread(&m_nWidth, sizeof(int), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// cǂݍ
	if (fread(&m_nHeight, sizeof(int), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// Cǂݍ
	if (fread(&m_wLayer, sizeof(WORD), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// Fǂݍ
	if (fread(&m_btTransColor, sizeof(BYTE), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// J[pbg̓ǂݍ
	if (fread(&m_acolPalette, 3, 256, fp) != 256) {
		fclose(fp);
		return EDGELIB_NOT_EDGE_FILE;
	}

	// f[^WJȂǂݍ
	CEdgeCompress Compressor;
	int nImageSize = m_nWidth * m_nHeight;

	char szTemp[CEdgeLayer::NAME_MAX];
	bool bShow;
	int i;
	for (i = 0; i < m_wLayer; i++) {
		
		// C擾
		if (fread(szTemp, CEdgeLayer::NAME_MAX, 1, fp) != 1) {
			fclose(fp);
			return EDGELIB_NOT_EDGE_FILE;
		}

		// \E\Ƃ
		if (fread(&bShow, 1, 1, fp) != 1) {
			fclose(fp);
			return EDGELIB_NOT_EDGE_FILE;
		}

		// C[WWJȂǂݍ
		BYTE* pbtImage = new BYTE[nImageSize];
		if (Compressor.Read(fp, pbtImage, nImageSize) == false) {
			delete pbtImage;
			fclose(fp);
			return EDGELIB_NOT_EDGE_FILE;
		}

		// Cǉ
		CEdgeLayer* pLayer = new CEdgeLayer();
		pLayer->SetName(szTemp);
		pLayer->SetShow(bShow);
		pLayer->SetImage(pbtImage, true);
		m_apLayer[i] = pLayer;
	}

	fclose(fp);

	return EDGELIB_OK;
}

// pbg擾
CEdgeColor* CEdgeFile::GetPalette()
{
	return m_acolPalette;
}

// Cǉ
int CEdgeFile::AddLayer(BYTE *pbtImage, const char *pszName, bool bShow)
{
	// G[`FbN
	if (pbtImage == NULL) {
		return EDGELIB_IMAGE_NULL;
	}

	if (m_wLayer >= LAYER_MAX) {
		return EDGELIB_LAYER_FULL;
	}

	CEdgeLayer* pLayer = new CEdgeLayer();
	pLayer->SetImage(pbtImage, false);
	if (pszName) {
		pLayer->SetName(pszName);
	}
	pLayer->SetShow(bShow);

	m_apLayer[m_wLayer++] = pLayer;

	return EDGELIB_OK;
}

// C擾
WORD CEdgeFile::GetLayerCount()
{
	return m_wLayer;
}

// 擾
int CEdgeFile::GetWidth()
{
	return m_nWidth;
}

// c擾
int CEdgeFile::GetHeight()
{
	return m_nHeight;
}

// C擾
CEdgeLayer* CEdgeFile::GetLayer(WORD wIndex)
{
	if (wIndex < 0 || wIndex >= m_wLayer) {
		return NULL;
	}
	return m_apLayer[wIndex];
}

// pbgݒ
void CEdgeFile::SetPalette(CEdgeColor *pcolPalette)
{
	CopyMemory(m_acolPalette, pcolPalette, 3 * 256);
}

// Fݒ
void CEdgeFile::SetTransColor(BYTE btColor)
{
	m_btTransColor = btColor;
}

// os
int CEdgeFile::Write(const char *pszFilename)
{
	FILE* fp = fopen(pszFilename, "wb");
	if (fp==NULL) {
		return EDGELIB_CANNOT_OPEN;
	}

	// t@Cwb_[o
	char szHeader[10] = "EDGE";
	if (fwrite(szHeader, 1, 10, fp) != 10) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// o
	if (fwrite(&m_nWidth, sizeof(int), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// co
	if (fwrite(&m_nHeight, sizeof(int), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// Co
	if (fwrite(&m_wLayer, sizeof(WORD), 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// Fo
	if (fwrite(&m_btTransColor, 1, 1, fp) != 1) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// J[pbgo
	if (fwrite(m_acolPalette, 3, 256, fp) != 256) {
		fclose(fp);
		return EDGELIB_CANNOT_WRITE;
	}

	// eCۑ
	CEdgeCompress EdgeComp;
	int i;
	int nImageSize = m_nWidth * m_nHeight;
	for (i = 0; i < m_wLayer; i++) {

		// Cۑ
		if (fwrite(m_apLayer[i]->GetName(), 1, CEdgeLayer::NAME_MAX, fp) != CEdgeLayer::NAME_MAX) {
			fclose(fp);
			return EDGELIB_CANNOT_WRITE;
		}

		// \\ۑ
		bool bShow = m_apLayer[i]->IsShow();
		if (fwrite(&bShow, 1, 1, fp) != 1) {
			fclose(fp);
			return EDGELIB_CANNOT_WRITE;
		}
		
		if (!EdgeComp.Write(fp, m_apLayer[i]->GetImage(), nImageSize)) {
			fclose(fp);
			return EDGELIB_CANNOT_WRITE;
		}
	}

	fclose(fp);

	return EDGELIB_OK;
}
